@@ -5,14 +5,22 @@ import pathlib
 import json
 import warnings
 from pathlib import Path
-from polly.errors import paramException
+from polly.errors import (
+    paramException,
+    extract_error_message_details,
+    UnauthorizedException,
+    EmptyPayloadException,
+    InvalidSchemaJsonException,
+    InvalidSyntaxForRequestException,
+)
 
 # import pandas as pd
 import polly.constants as const
 from cryptography.fernet import Fernet
 from polly.errors import error_handler, ValidationError
 import polly.helpers as helpers
-
+import polly.http_response_codes as http_codes
+from polly import application_error_info as app_err_info
 
 dataset_level_metadata_files_not_uploaded = []
 data_files_whose_metadata_failed_validation = []
@@ -46,7 +54,7 @@ def parameter_check_for_repo_id(repo_id):
     elif type(repo_id) != str and type(repo_id) != int:
         raise paramException(
             title="Param Error",
-            detail="repo_id should be str or int",
+            detail=f"repo_id, {repo_id} should be str or int",
         )
 
 
@@ -64,6 +72,55 @@ def make_repo_id_string(repo_id: int) -> str:
     return repo_id
 
 
+def schema_param_check(repo_key: str, body: dict):
+    """Do sanity checks for the parameters passed in schema functions
+
+    Args:
+        repo_key (str): _description_
+        body (dict): _description_
+    Raises:
+        paramError: In case of any issue with the parameter
+    """
+    try:
+        if not body or not isinstance(body, dict):
+            raise paramException(
+                title="Param Error",
+                detail="body should not be empty and it should be of type dict",
+            )
+        parameter_check_for_repo_id(repo_key)
+    except Exception as err:
+        raise err
+
+
+def create_move_data_payload(
+    payload_datasets: list, src_repo_key: str, dest_repo_key: str, priority: str
+) -> dict:
+    """_summary_
+
+    Args:
+        payload_datasets (list): _description_
+        src_repo_key (str): _description_
+        dest_repo_key (str): _description_
+        priority (str): _description_
+    Returns:
+        Dict : Dictionary of the move data payload
+    """
+    move_data_payload = {}
+    move_data_payload["data"] = {}
+    move_data_payload["data"]["type"] = "ingestion-transaction"
+    move_data_payload["data"]["attributes"] = {}
+    move_data_payload["data"]["attributes"]["datasets"] = payload_datasets
+    move_data_payload["data"]["attributes"]["source_repo_id"] = src_repo_key
+    move_data_payload["data"]["attributes"]["destination_repo_id"] = dest_repo_key
+    move_data_payload["data"]["attributes"]["priority"] = priority
+    move_data_payload["data"]["attributes"]["flags"] = {}
+    move_data_payload["data"]["attributes"]["flags"]["file_metadata"] = "true"
+    move_data_payload["data"]["attributes"]["flags"]["col_metadata"] = "true"
+    move_data_payload["data"]["attributes"]["flags"]["row_metadata"] = "true"
+    move_data_payload["data"]["attributes"]["flags"]["data_required"] = "true"
+    return move_data_payload
+
+
 def update_dataset_parameter_check(source_folder_path: dict):
     """Check update dataset parameter check
 
@@ -98,8 +155,6 @@ def update_dataset_parameter_check(source_folder_path: dict):
 # or is there additional key present in source_folder_path
 # Ex - https://github.com/ElucidataInc/polly-python-code/pull/257/files#r1048116264
 # check the above link
-
-
 def add_dataset_parameter_check(source_folder_path: dict):
     """Check update dataset parameter check
 
@@ -642,6 +697,24 @@ def get_file_name_without_suffixes(data_file: str) -> str:
         else:
             # single word extension with `.`'s in file which is accepted
             data_file_name = pathlib.Path(data_file).stem
+    elif len(file_ext) == 0:
+        # this case will never arise when we are passing files from add or
+        # update function i.e. data and metadata files
+        # Because this check is applied in the parameter check
+
+        # this case will arise if somebody has ingested a file without extension
+        # through quilt. In that case while we are updating either data or
+        # metadata files in the repo, we are fetching all the files of that repo
+        # there a file can exist which does not have extension, for that edgecase
+        # this check is helpful
+        warnings.formatwarning = lambda msg, *args, **kwargs: f"WARNING: {msg}\n"
+        warnings.warn(
+            f"{data_file}"
+            + " does not have an extension. "
+            + "This file is invalid. Please delete this file. "
+            + "For any questions, please reach out to polly.support@elucidata.io."
+        )
+        data_file_name = ""
     return data_file_name
 
 
@@ -653,6 +726,30 @@ def parameter_check_for_priority(priority: str):
         )
 
 
+def move_data_params_check(
+    src_repo_key: str, dest_repo_key: str, dataset_ids: list, priority: str
+):
+    """Check move data params
+
+    Args:
+        src_repo_key (str/int): source repo id
+        dest_repo_key (str/int): destination repo id
+        dataset_ids (list): List of dataset ids
+        priority (str): priority of the operation
+    """
+    try:
+        parameter_check_for_repo_id(src_repo_key)
+        parameter_check_for_repo_id(dest_repo_key)
+        parameter_check_for_priority(priority)
+        if not dataset_ids or not isinstance(dataset_ids, list):
+            raise paramException(
+                title="param error",
+                detail="dataset_ids should not be empty and dataset_ids should be in list format.",
+            )
+    except Exception as err:
+        raise err
+
+
 def metadata_files_for_upload(source_metadata_path: str) -> list:
     """Find List of all the metadata files to be uploaded
 
@@ -821,6 +918,8 @@ def check_create_omixatlas_parameters(
     image_url: str,
     components: list,
     category: str,
+    data_type: str,
+    org_id: str,
 ):
     """Sanity check for Parameters passed in Create Method
     Args:
@@ -830,24 +929,110 @@ def check_create_omixatlas_parameters(
         image_url (str): image url provided for the icon for the omixatlas
         components (list): components to be added in the omixatlas
         category (str): category definition of the omixatlas
+        data_type(str): datatype of the omixatlas. By default it is None
+        org_id(str): org id of the Organisation. By default it is empty
     """
-    str_params = [display_name, description, repo_name, image_url]
-    for param in str_params:
-        if not isinstance(param, str):
+    try:
+        str_params = [display_name, description, repo_name, image_url]
+        for param in str_params:
+            if not isinstance(param, str):
+                raise paramException(
+                    title="Param Error", detail=f"{param} should be a string"
+                )
+        check_components_parameter(components)
+
+        if not isinstance(category, str) or (
+            category not in const.OMIXATLAS_CATEGORY_VALS
+        ):
             raise paramException(
-                title="Param Error", detail=f"{param} should be a string"
+                title="Param Error",
+                detail=f"category should be a string and its value must be one of {const.OMIXATLAS_CATEGORY_VALS}",
+            )
+        check_org_id(org_id)
+        check_data_type_parameter(data_type)
+    except Exception as err:
+        raise err
+
+
+def check_org_id(org_id: str):
+    """Sanity check for org id
+
+    Args:
+        org_id (str): org id of the Organisation. By default it is empty
+    """
+    if org_id:
+        if not isinstance(org_id, str):
+            raise paramException(
+                title="Param Error",
+                detail="org id should be a string.",
             )
 
+
+def check_data_type_parameter(data_type: str):
+    """Check for data type parameter
+
+    Args:
+        data_type (str): datatype of the omixatlas. By default it is None
+    """
+    if data_type:
+        if not isinstance(data_type, str) or (
+            data_type not in const.OMIXATLAS_DATA_TYPE_VALS
+        ):
+            raise paramException(
+                title="Param Error",
+                detail=(
+                    "data_type should be a string and its value must be one of "
+                    + f"{const.OMIXATLAS_DATA_TYPE_VALS}"
+                ),
+            )
+
+
+def check_components_parameter(components: list):
+    """Check components parameter
+
+    Args:
+        components (list): components to be added in the omixatlas
+    """
     if not isinstance(components, list):
         raise paramException(
             title="Param Error", detail=f"{components} should be a list"
         )
 
-    if not isinstance(category, str) or (category not in const.OMIXATLAS_CATEGORY_VALS):
-        raise paramException(
-            title="Param Error",
-            detail=f"{category} should be a string and its value must be one of {const.OMIXATLAS_CATEGORY_VALS}",
-        )
+
+def check_update_omixatlas_parameters(
+    display_name: str, description: str, repo_key: str, image_url: str, components: list
+):
+    """Sanity check for Parameters passed in Update Method
+    Args:
+        display_name (str): Display name of the Omixatlas
+        description (str): Description of the Omixatlas
+        repo_name (str): proposed repo name for the omixatlas
+        image_url (str): image url provided for the icon for the omixatlas
+        components (list): components to be added in the omixatlas
+        data_type(str): datatype of the omixatlas. By default it is None
+    """
+    try:
+        parameter_check_for_repo_id(repo_key)
+        # this can be refactored using args -> just passing **args into the function
+        # have a discussion on this
+        if not display_name and not description and not image_url and not components:
+            raise paramException(
+                title="Param Error",
+                detail=(
+                    "No parameters passed to update, please pass at least one of the following"
+                    + " params [display_name, description, image_url, components, data_type]."
+                ),
+            )
+
+        str_params = [display_name, description, image_url]
+        for param in str_params:
+            if not isinstance(param, str):
+                raise paramException(
+                    title="Param Error", detail=f"{param} should be a string"
+                )
+        check_components_parameter(components)
+    except Exception as err:
+        raise err
 
 
 def verify_repo_identifier(repo_identifier: str):
@@ -1022,3 +1207,72 @@ def check_if_destination_folder_correct(
             corresponding_data_file_names, required_file_name, destination_folder_path
         )
     return is_dest_folder_correct
+
+
+def check_params_for_get_metadata(repo_key, dataset_id, table_name):
+    """
+    Args:
+        repo_key(str): repo_name/repo_id of the repository.
+        dataset_id(str): dataset_id of the dataset.
+        table_name(str): table name for the desired metadata, 'samples' supported for now.
+    """
+    if not (repo_key and (isinstance(repo_key, str) or isinstance(repo_key, int))):
+        raise paramException(
+            title="Param Error",
+            detail="Argument 'repo_key' is either empty or invalid. \
+It should either be a string or an integer. Please try again.",
+        )
+    if not (dataset_id and isinstance(dataset_id, str)):
+        raise paramException(
+            title="Param Error",
+            detail="Argument 'dataset_id' is either empty or invalid. It should be a string. Please try again.",
+        )
+    if not (table_name and isinstance(table_name, str)):
+        raise paramException(
+            title="Param Error",
+            detail="Argument 'table_name' is either empty or invalid. It should be a string. Please try again.",
+        )
+    if table_name.lower() not in const.TABLE_NAME_SAMPLE_LEVEL_INDEX_MAP:
+        raise paramException(
+            title="Param Error",
+            detail=const.ERROR_MSG_GET_METADATA,
+        )
+
+
+def param_check_download_dataset(
+    repo_key: str, dataset_ids: list, folder_path: str
+) -> None:
+    if not (repo_key and isinstance(repo_key, str)):
+        raise paramException(
+            title="Param Error",
+            detail="repo_key (either id or name) is required and should be a string",
+        )
+    if not (dataset_ids and isinstance(dataset_ids, list)):
+        raise paramException(
+            title="Param Error",
+            detail="dataset_ids is required and should be a list of dataset_ids as strings",
+        )
+    if (not isinstance(folder_path, str)) or (not os.path.isdir(folder_path)):
+        raise paramException(
+            title="Param Error",
+            detail="folder_path if provided should be a string and a valid folder path.",
+        )
+
+
+def handle_elastic_discover_api_error(response):
+    if response.status_code == http_codes.UNAUTHORIZED:
+        raise UnauthorizedException("User is unauthorized to access this")
+    elif response.status_code == http_codes.BAD_REQUEST:
+        title, details = extract_error_message_details(response)
+        if title == app_err_info.EMPTY_PAYLOAD_CODE:
+            raise EmptyPayloadException()
+        elif app_err_info.INVALID_MODEL_NAME_TITLE in title:
+            raise InvalidSyntaxForRequestException()
+    elif response.status_code == http_codes.INTERNAL_SERVER_ERROR:
+        raise InvalidSchemaJsonException()
+    else:
+        (
+            title,
+            details,
+        ) = extract_error_message_details(response)
+        raise Exception("Exception Occurred :" + str(details))
